@@ -2,6 +2,7 @@ from typing import Optional, Union, List
 from datetime import datetime
 from concurrent.futures import ThreadPoolExecutor
 import itertools
+import traceback
 
 import numpy as np
 import pandas as pd
@@ -16,6 +17,8 @@ __all__ = [
     "get_batches",
     "update_batch_values",
     "update_batch_feature_values",
+    "update_batch_features_and_values",
+    "update_vector_batch_values",
 ]
 
 
@@ -93,7 +96,7 @@ def get_batch_type_details(
     Parameters
     ----------
     batch_type_id: str
-        The id of the batch type to be retrieved
+        The id of the batch type to be retrieved.
     api_credentials: OIAnalyticsAPICredentials, optional
         The credentials to use to query the API. If None, previously set default credentials are used.
 
@@ -397,6 +400,26 @@ def update_batch_values(
     batch_id_index_name: str = "batch_id",
     api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
 ):
+    """
+
+    Parameters
+    ----------
+    batch_type_id: str
+        The id of the batch type to be retrieved
+    data: pd.Series or pd.DataFrame
+        Object containing the data to be uploaded, where the index contains the batches ids and the
+    unit_ids: dict[str, str], optional
+        Dictionary with the unit-ids of data to be uploaded.
+        Each key is a data-id that points to the corresponding unit-id.
+    batch_id_index_name: str, default 'batch_id'
+        The name of the Series's or DataFrame's index
+    api_credentials: OIAnalyticsAPICredentials, optional
+        The credentials to use to query the API. If None, previously set default credentials are used.
+
+    Returns
+    -------
+
+    """
     # Init
     if unit_ids is None:
         unit_ids = {}
@@ -433,7 +456,12 @@ def update_batch_values(
     )
 
     with ThreadPoolExecutor() as pool:
-        pool.map(send_value, batch_values)
+        updates = pool.map(send_value, batch_values)
+
+    try:
+        print(f"{len(list(updates))} batch values sent to OIAnalytics")
+    except Exception:
+        print(f"Error during batch values update:\n{traceback.format_exc()}")
 
 
 def update_batch_feature_values(
@@ -442,6 +470,27 @@ def update_batch_feature_values(
     batch_id_index_name: str = "batch_id",
     api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
 ):
+    """
+    Insert/update batches features values
+    Parameters
+    ----------
+    batch_type_id: str
+        The id of the batch type to be retrieved.
+    data: pd.Series or pd.DataFrame
+        Object containing the features values to be uploaded:
+        - The index contains the batch-ids;
+        - If the object is a DataFrame, the columns contains the features names;
+        - If the object is a Series, the Series' name is the feature name being updated
+        - The data contains the features values.
+    batch_id_index_name: str, default 'batch_id'
+        The name of the Dataframe's or Series's index.
+    api_credentials: OIAnalyticsAPICredentials, optional
+        The credentials to use to query the API. If None, previously set default credentials are used.
+
+    Returns
+    -------
+
+    """
     # Init
     if isinstance(data, pd.Series):
         data = pd.DataFrame(data)
@@ -475,3 +524,145 @@ def update_batch_feature_values(
 
     with ThreadPoolExecutor() as pool:
         pool.map(send_value, batch_values)
+
+
+def update_batch_features_and_values(
+    batch_type: str,
+    data: pd.DataFrame,
+    feature_columns: List[str],
+    units_ids: Optional[dict[str, str]] = None,
+    use_external_reference: bool = False,
+    api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
+) -> None:
+    """
+    It updates many batches values and features values at once
+    Parameters
+    ----------
+    batch_type: str
+        The id of the batch type to be retrieved.
+    data: pd.DataFrame
+        Object containing the features values and values to be uploaded:
+        - The index contains the batch-ids;
+        - The columns contains the features-ids and data-ids;
+        - The data contains the features values.
+    feature_columns: array of str
+        List containing the names of the Dataframe's columns corresponding to the features-ids.
+    units_ids: dict[str, str], optional
+        Dictionary with the unit-ids of data to be uploaded.
+        Each key is a data-id that points to the corresponding unit-id.
+    use_external_reference: bool, default False
+        If True, the data are named using their external reference; if False, the OIAnalytics reference is used.
+    api_credentials: OIAnalyticsAPICredentials, optional
+        The credentials to use to query the API. If None, previously set default credentials are used.
+
+    Returns
+    -------
+
+    """
+    payload = []
+    for index in data.index:
+        payload.append(
+            {
+                "batchId": index,
+                "batchTagCommands": [
+                    {"batchTagKeyId": tag, "batchTagValueValue": data.loc[index, tag],}
+                    for tag in feature_columns
+                    if data.loc[index, tag] is not None
+                ],
+                "batchValueCommands": [
+                    {
+                        "dataId": data_id,
+                        "value": data.loc[index, data_id],
+                        "unitId": units_ids.get(data_id),
+                    }
+                    for data_id in [
+                        col for col in data.columns if col not in feature_columns
+                    ]
+                    if data.loc[index, data_id] is not None
+                ],
+            }
+        )
+
+    endpoints.batches.update_batch_features_and_values(
+        batch_type=batch_type,
+        data=payload,
+        use_external_reference=use_external_reference,
+        api_credentials=api_credentials,
+    )
+
+
+def update_vector_batch_values(
+    data: Union[pd.DataFrame, List[pd.DataFrame]],
+    data_reference: Union[str, List[str]],
+    index_units: Optional[dict] = None,
+    values_units: Optional[dict] = None,
+    batch_id_index_name: str = "batch_id",
+    use_external_reference: bool = False,
+    api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
+):
+    """
+    Insert time values stored in a DataFrame through the OIAnalytics API
+
+    Parameters
+    ----------
+    data: list of dictionaries
+        List where each element has a 'dataReference' as a key and a dataframe as a value.
+    data_reference: string or list of strings
+        The unique data reference for the data being inserted.
+    index_units: dictionary, optional
+        A dictionary indexed by data reference, specifying the values in which it is sent.
+    values_units: dictionary, optional
+        A dictionary indexed by data reference, specifying the values in which it is sent.
+    batch_id_index_name: string, default 'batch_ids'
+        The name of the index of the DataFrame(s)
+    use_external_reference: bool, default False
+        If True, the data are named using their external reference; if False, the OIAnalytics reference is used.
+    api_credentials: OIAnalyticsAPICredentials, optional
+        The credentials to use to query the API. If None, previously set default credentials are used.
+
+    Returns
+    -------
+    A dictionary of the response from the API, containing the data insert report
+    """
+
+    # Init 'index_units'
+    if index_units is None:
+        index_units = {}
+
+    # Init 'values_units'
+    if values_units is None:
+        values_units = {}
+
+    # Init list of data and list of data_reference
+    if isinstance(data, pd.DataFrame):
+        data = [data.copy()]
+        data_reference = [data_reference]
+
+    # Build DTO
+    payload = []
+    for reference, df in zip(data_reference, data):
+        try:
+            df.index = df.index.get_level_values(batch_id_index_name)
+        except KeyError:
+            raise KeyError("The dataframe must have an index level named 'batch_id'")
+        # Build payload for the data
+        payload.append(
+            {
+                "dataReference": reference,
+                "indexUnit": index_units.get(reference, None),
+                "valueUnit": values_units.get(reference, None),
+                "batchIds": df.index.tolist(),
+                "indexes": df.columns.tolist(),
+                "values": df.to_numpy().tolist(),
+            }
+        )
+
+    # Query endpoint
+    response = endpoints.batches.update_vector_batch_values(
+        data=payload,
+        use_external_reference=use_external_reference,
+        api_credentials=api_credentials,
+    )
+
+    # Output
+    return response
