@@ -937,7 +937,7 @@ def insert_vector_time_values(
     index_units: Optional[dict] = None,
     values_units: Optional[dict] = None,
     use_external_reference: bool = False,
-    timestamps_index_name: str = "timestamp",
+    timestamp_index_name: str = "timestamp",
     api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
 ):
     """
@@ -955,7 +955,7 @@ def insert_vector_time_values(
         A dictionary indexed by data reference, specifying the values in which it is sent
     use_external_reference: bool, default False
         Whether the data are named using their external reference (else the OIAnalytics reference is used).
-    timestamps_index_name: str, default 'timestamps'
+    timestamp_index_name: str, default 'timestamp'
         Name of the index level containing the timestamps
     api_credentials: OIAnalyticsAPICredentials, optional
         The credentials to use to query the API. If None, previously set default credentials are used.
@@ -983,10 +983,10 @@ def insert_vector_time_values(
     for reference, df in zip(data_reference, data):
         try:
             timestamps = (
-                df.index.get_level_values(timestamps_index_name)
+                df.index.get_level_values(timestamp_index_name)
                 .to_series()
                 .apply(utils.get_zulu_isoformat)
-                .rename(timestamps_index_name)
+                .rename(timestamp_index_name)
             )
         except KeyError:
             raise KeyError(
