@@ -9,6 +9,7 @@ import pandas as pd
 
 __all__ = [
     "get_zulu_isoformat",
+    "floor_date",
     "get_iso_period",
     "expand_dataframe_column",
     "concat_pages_to_dataframe",
@@ -26,6 +27,39 @@ def get_zulu_isoformat(date: Union[str, datetime]):
     return date.astimezone(timezone.utc).isoformat().replace("+00:00", "Z")
 
 
+def floor_date(date, period, pivot_date="2018-01-01"):
+    """
+    Floors a date using a period and a pivot date, in order to stabilize the timestamps.
+    It is floored to the closest previous datetime being 'pivot_date' + n * 'period'.
+    The default pivot date is '2018-01-01' since it was a monday.
+
+    Parameters
+    ----------
+    date: datetime
+        The date to be floored. Must have a timezone (UTC will ease the setting of the 'pivot_date')
+    period: str
+        ISO period used to floor the 'date'
+    pivot_date: datetime or str
+        The date used as a reference for the floor operation.
+        If a string is provided, it is supposed to be in UTC.
+
+    Returns
+    -------
+    The closest previous datetime being 'pivot_date' + n * 'period'
+    """
+
+    # Make the pivot date an actual datetime in UTC
+    if isinstance(pivot_date, str):
+        ref_date = pd.to_datetime(pivot_date, utc=True)
+    else:
+        ref_date = pivot_date
+
+    delta = date - ref_date
+    nb_period = delta // pd.Timedelta(period)
+
+    return pd.Timedelta(period) * nb_period + ref_date
+
+
 def get_iso_period(period: Optional[Union[str, timedelta, pd.Timedelta]]):
     if period is None:
         return None
