@@ -121,9 +121,7 @@ class TestModelExecution(_dtos.ModelExecution):
             trigger = _dtos.ManualOnlyTrigger(type="manual-only")
             execution_context_type = "manual"
         else:
-            raise ValueError(
-                f"{trigger_type} is not valid for 'trigger_type'"
-            )
+            raise ValueError(f"{trigger_type} is not valid for 'trigger_type'")
 
         # Init
         super().__init__(
